import asyncio

import uvloop

asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

import multiprocessing
import os
from concurrent.futures.thread import ThreadPoolExecutor
from contextlib import ContextDecorator

from util.daemon import PidFile, DaemonContext
from .log import wrap


class EnteringLoop(ContextDecorator):
    def __init__(self, name, log_dir, log_level, is_forground=True, pid_path=None):
        self.name = name
        self.log_dir = log_dir
        self.log_level = log_level
        self.is_forground = is_forground
        self.pid_path = pid_path
        self.wrapper = None
    
    def __enter__(self):
        if self.is_forground:
            wrap(self.name, level=self.log_level, stderr=True)
            self.wrapper = PidFile(self.pid_path)
        else:
            keep_fds = wrap(self.name, log_dir=self.log_dir, level=self.log_level)
            self.wrapper = DaemonContext(
                working_directory=os.getcwd(),
                files_preserve=keep_fds,
                pidfile=PidFile(self.pid_path)
            )
        self.wrapper.__enter__()
        loop = asyncio.new_event_loop()
        loop.set_default_executor(ThreadPoolExecutor(max_workers=multiprocessing.cpu_count() * 2 + 1))
        
        asyncio.set_event_loop(loop)
        self.loop = loop
        return loop
    
    def __exit__(self, exc_type, exc, exc_tb):
        if self.loop is not None:
            self.loop.close()
        if self.wrapper is not None:
            self.wrapper.__exit__(exc_type, exc, exc_tb)
