import errno

import os


def to_module_path(*rest_path):
    """
    현재 모듈위치에서 파일 경로 계산
    :param rest_path:
    :return:
    """
    return os.path.abspath(os.path.normpath(os.path.join(*rest_path)))


def makedirs(path):
    """
    python2에서도 makedirs 수행시 이미경로가 있을 경우 그냥 지나가는 함수
    :param path:
    :return:
    """
    try:
        os.makedirs(path)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise e

def find_executable_of_specify(find_exe_name, location=None):
    """
    지정된 실행파일이 정상인지 보거나 지정되지 않았을 경우 PATH environment에서 실행파일 찾기
    :param find_exe_name:
    :param location:
    :return:
    """

    # 람다 : 파일인지 ? 실행 파일인지?
    def is_executable(fpath):
        return os.path.isfile(
            fpath) and os.access(fpath, os.X_OK)

    if location is None:
        # 경로가 지정되지 않았으면 PATH environ에서 뒤져본다.
        for pathItem in os.environ["PATH"].split(os.pathsep):
            pathItem = pathItem.strip('"')
            exe_path = os.path.join(pathItem, find_exe_name)
            if is_executable(exe_path):
                location = exe_path
                break
        # 없으면 끝
        if location is None:
            raise IOError("error: cannot find %s from PATH!" % find_exe_name)
    elif not is_executable(to_module_path(location)):
        raise IOError("error: %s is not executable!" % find_exe_name)
    else:
        location = to_module_path(location)
    return location

