import logging
from logging import handlers as loggingHandlers
from os import path

# 로거 설정
from .file import to_module_path,makedirs


def wrap(name, level=logging.INFO, log_dir=None, stderr=False):
    keep_fds = []
    if log_dir is not None:
        makedirs(to_module_path(log_dir))
    formatter = logging.Formatter(
        '[%(asctime)s] [%(process)d] [%(levelname)s] (%(name)s) > %(message)s',
        datefmt="%Y%m%d%H%M%S %z"
    )
    
    if stderr:
        log_handler = logging.StreamHandler()
    else:
        log_handler = loggingHandlers.TimedRotatingFileHandler(
            path.join(log_dir, '%s.log' % name),
            when='d',
            backupCount=3
        )
        keep_fds.append(log_handler.stream.fileno())
    log_handler.setFormatter(formatter)
    
    root_logger = logging.root
    root_logger.setLevel(level)
    root_logger.addHandler(log_handler)
    
    return keep_fds
